@@ -4,8 +4,14 @@ class Admin::AdminUsersController < Admin::BaseController
   before_filter :load_admin
  
   index.before do
-    @admin_users = AdminUser.find(:all,
-                                  :order => 'position')
+    if session[:admin_user].master_shop?
+      @admin_users = AdminUser.find(:all,
+                                    :order => 'position')
+    else
+      @admin_users = AdminUser.find(:all, 
+                                    :conditions => ["retailer_id = ?", session[:admin_user].retailer_id], 
+                                    :order => 'position')
+    end
   end
 
   [create, update].each do |action|
@@ -14,6 +20,12 @@ class Admin::AdminUsersController < Admin::BaseController
     end
   end
 
+  edit.before do
+    unless session[:admin_user].master_shop?
+      raise ActiveRecord::RecordNotFound if AdminUser.find(params[:id]).retailer_id != session[:admin_user].retailer_id
+    end
+  end
+
   def up
     super
     redirect_to :action => "index"
