@@ -2,10 +2,10 @@
 require 'timeout'
 require 'open-uri'
 class CartController < BaseController
-  before_filter :cart_check, :only => [:temporary_shipping,:shipping, :purchase,:purchase2, :confirm, :complete]
-  before_filter :login_divaricate ,:only =>[:purchase,:purchase2,:confirm, :complete]
+  before_filter :cart_check, :only => [:temporary_shipping,:shipping, :purchase,:purchase2, :confirm, :complete, :delivery, :delivery2]
+  before_filter :login_divaricate ,:only =>[:purchase,:purchase2,:confirm, :complete, :delivery]
   before_filter :login_check, :only => [:shipping]
-  before_filter :force_post, :only => [:purchase,:purchase2,:confirm, :complete]
+  before_filter :force_post, :only => [:delivery, :purchase,:purchase2,:confirm, :complete]
   after_filter :save_carts
   before_filter :verify_session_token, :except => :select_delivery_time
   
@@ -134,9 +134,8 @@ class CartController < BaseController
         convert(params[:order_delivery])
      end
   end
-  
-  # Order を作る
-  def purchase
+ 
+  def delivery
     cookies.delete :back_from_deliv if cookies[:back_from_deliv]
 
     #2.配送先の情報を取ってくる
@@ -184,40 +183,66 @@ class CartController < BaseController
     # 住所を取得できないと、この先困るので、どこかに飛ばす
     return redirect_to(:action => 'show') unless @delivery_address
 
-    # 確認ページから帰ってきた時は params[:order_delivery] があるはず
-    @order_delivery = OrderDelivery.new(params[:order_delivery])
-    #確認ページから帰ってきた時、ポイント使用有無ラジオボタン維持
-    if @login_customer && params[:back] == "1"
-      if !@order_delivery.use_point.blank? && @order_delivery.use_point > 0
-        @point_check = true
-      else
-        @point_check = false
-        @order_delivery.use_point = nil
-      end        
+    @order_deliveries = Hash.new
+    unless params[:order_deliveries].nil?
+      params[:order_deliveries].each do |key, order_delivery|
+        @order_deliveries[key] = OrderDelivery.new(order_delivery)
+      end
     end
-    # 「戻る」以外の時代入
-    @order_delivery.address_select ||= params[:address_select].to_i
-    if @order_delivery.deliv_zipcode01.blank?
-      @order_delivery.set_delivery_address(@delivery_address)
+    if @order_deliveries.empty?
+      @carts.map(&:product_style).map(&:product).map(&:retailer).each do |retailer|
+        od = OrderDelivery.new
+        od.set_delivery_address(@delivery_address)
+        @order_deliveries[retailer.id] = od
+      end
     end
-
-    if params[:order_delivery]
-      @order_delivery.target_columns = params[:order_delivery].keys.map(&:to_s)
+    @delivery_traders = Hash.new
+    @carts.map(&:product_style).map(&:product).map(&:retailer).each do |retailer|
+      @delivery_traders[retailer.id] = select_delivery_trader_with_retailer_id(retailer.id)
     end
-    #@payments = @order_delivery.payment_candidates(@cart_price)
-    #配送時間取得・表示のAJAXははお支払方法押下時のみ行い、戻るボタンで戻る時、配送時間表示がおかしくないため、
-    #支払方法を強制にクリア。モバイルはAJAXを使っていないので、そのまま
-    @order_delivery.payment = nil unless request.mobile?
-
-    # 非会員フラグ追加
-    # 非会員購入時の顧客情報セット
     if @not_login_customer
-      @order_delivery.set_customer(@temporary_customer)
+      @order_deliveries.each do |key, order_delivery|
+        order_delivery.set_customer(@temporary_customer)
+      end
     end
+    render :action => 'delivery'
+  end
+
+  #TODO テストケースの作成
+  def delivery2
+    @order_deliveries = Hash.new
+    unless params[:order_deliveries].nil?
+      params[:order_deliveries].each do |key, order_delivery|
+        @order_deliveries[key] = OrderDelivery.new(order_delivery)
+      end
+    else
+      #error
+    end
+    @delivery_time_options = Hash.new
+    @order_deliveries.each do |retailer_id, od|
+      delivery_trader_id = od.delivery_trader_id
+      option = select_delivery_time_with_delivery_trader_id(delivery_trader_id)
+      @delivery_time_options[retailer_id] = option
+    end
+    render :action => 'delivery2'
+  end
+
+  # Order を作る
+  def purchase
+    @order_deliveries = Hash.new
+    unless params[:order_deliveries].nil?
+      params[:order_deliveries].each do |key, order_delivery|
+        @order_deliveries[key] = OrderDelivery.new(order_delivery)
+      end
+    else
+      #error
+    end
+    
     render :action => 'purchase'
   end
   
   #モバイルお届け時間選択
+  #現在未使用
   def purchase2
     @order_delivery = OrderDelivery.new(params[:order_delivery])
     unless @order_delivery.valid?
@@ -257,7 +282,14 @@ class CartController < BaseController
     select_delivery_time
     @order_delivery.target_columns = params[:order_delivery].keys.map(&:to_s)
   end
-  
+ 
+  def select_delivery_time_with_delivery_trader_id_ajax
+    delivery_trader_id = params[:delivery_trader_id]
+    @options = select_delivery_time_with_delivery_trader_id(delivery_trader_id)
+    render :layout => false unless request.mobile?
+  end
+
+
   # AJAXお届け時間取得
   def select_delivery_time
     h = params[:order_delivery] || params
@@ -275,55 +307,53 @@ class CartController < BaseController
 
   #確認画面へ
   def confirm
-    init_order_delivery
-    unless @order_delivery.valid?
-      @order_delivery.payment = nil unless request.mobile?
-      if params[:point_check] == "true"
-        @point_check = true
+    init_order_deliveries
+    @order_deliveries.each do |key, od|
+      unless od.valid?
+        render :action => 'purchase'
+        return
       end
-      render :action => 'purchase'
-      return
     end
-    
-    # ポイント
+
     if @login_customer
-      use_point = 0
-      if params[:point_check] == "true"
-        @point_check = true
-        use_point = @order_delivery.use_point.to_i
-        if use_point == 0
-           flash.now[:error] = '使用ポイントをご入力ください。 '
-           @order_delivery.payment = nil unless request.mobile?
-           render :action => 'purchase'
-           return
+      @all_use_point = 0
+      @order_deliveries.each do |retailer_id, od|
+        if params[:points][retailer_id][:point_check] == "true"
+          use_point = od.use_point.to_i
+          if use_point <= 0
+            flash.now[:error] = '使用ポイントをご入力ください。'
+            render :action => 'purchase'
+            return
+          end
+          if use_point > @cart_price_map[retailer_id.to_i].to_i
+            flash.now[:error] = 'ご利用ポイントがご購入金額を超えています。'
+            render :action => 'purchase'
+            return
+          end
+          @all_use_point = @all_use_point + use_point
+        else
+          od.use_point = 0
         end
-        # ポイントの使いすぎをチェック
-        if use_point > @cart_price
-          flash.now[:error] = 'ご利用ポイントがご購入金額を超えています。'
-          @order_delivery.payment = nil unless request.mobile?
-          render :action => 'purchase'
-          return
-        end
-        if use_point > @login_customer.point.to_i
+        if @all_use_point > @login_customer.point.to_i
           flash.now[:error] = 'ご利用ポイントが所持ポイントを超えています。'
-          @order_delivery.payment = nil unless request.mobile?
           render :action => 'purchase'
           return
-        end        
-      else
-        @point_check = false
-        @order_delivery.attributes = {:use_point => 0}
+        end
+        
+        od.attributes = {:add_point => total_points_each_cart(@carts_map[retailer_id.to_i])} 
       end
-      
       @cart_point = total_points
-      @point_after_operation = @login_customer.point.to_i - use_point + total_points
-      #add_point追加
-      @order_delivery.attributes = {:add_point => @cart_point}
+      @point_after_operation = @login_customer.point.to_i - @all_use_point + @cart_point
+      
     end
-    #ポイントのことで再計算はここにする
-    @order_delivery.calculate_charge!
-    @order_delivery.calculate_total!
     
+    @payment_total = 0
+    @order_deliveries.each do |retailer_id, od |
+      od.calculate_charge!
+      od.calculate_total!
+      @payment_total = @payment_total + od.payment_total
+    end
+
     @next = :complete
     render :action => 'confirm'
   end
@@ -334,48 +364,44 @@ class CartController < BaseController
       redirect_to :action => :show
       return
     end
-    # 受注を作って受注可能数を減らす
-          # 　非会員購入追加
-    if @not_login_customer
-      @order = Order.new
-    elsif @login_customer
-      @login_customer.point = params[:point_after_operation]
-      @order = @login_customer.orders.build
-    end
-    @order.received_at = DateTime.now
-    @order_delivery = @order.order_deliveries.build(params[:order_delivery])
-    
-    # 　非会員購入追加
-    if @login_customer
-      @order_delivery.set_customer(@login_customer)  
-    end
-  
-    ## ステータス
-    @order_delivery.status = OrderDelivery::JUTYUU
-    # 受注発注商品が一つでもあるか
-    product_styles = @carts.map(&:product_style)
-    #販売可能数で判断
-    if product_styles.any?{|ps| ps.orderable_count.to_i == 0}
-      # 販売開始日が未来の物が一つでもあれば予約
-      products = product_styles.map(&:product)
-      today = Date.today
-      if products.any?{|p| p.sale_start_at && p.sale_start_at > today}
-        @order_delivery.status = OrderDelivery::YOYAKU_UKETSUKE
+    @login_customer.point = params[:point_after_operation] if @login_customer
+    @orders = Hash.new
+    @order_deliveries = Hash.new
+    @order_details = Hash.new
+    ids = Array.new
+    params[:order_deliveries].each do |key, _od|
+      order = nil
+      if @not_login_customer
+        order = Order.new
       else
-        @order_delivery.status = OrderDelivery::JUTYUU_TOIAWASE
+        order = @login_customer.orders.build
+      end
+      order.retailer_id = key.to_i
+      order.received_at = DateTime.now
+      od = order.order_deliveries.build(_od)
+      od.set_customer(@login_customer) if @login_customer
+      od.status = OrderDelivery::JUTYUU
+      @orders[key] = order
+      @order_deliveries[key] = od
+      cart = @carts_map[key.to_i]
+      @order_details[key] = od.details_build_from_carts(cart)
+      od.calculate_charge!
+      od.calculate_total!
+      ids << @order_details[key].map{|o_d| o_d.product_style.product_id}
+    end  
+
+    @order_deliveries.each do |key, od|
+      unless od.valid? and @order_details[key].all?(&:valid?)
+        render :action => 'purchase'
+        return 
       end
     end
-    @carts.each do |cart|
-      cart.product_style.order(cart.quantity)
-    end
-    @order_details = @order_delivery.details_build_from_carts(@carts)
-    @order_delivery.calculate_charge!
-    @order_delivery.calculate_total!
 
-    unless @order_delivery.valid? and @order_details.all?(&:valid?)
+    if @order_deliveries.empty? or @order_details.empty?
       render :action => 'purchase'
       return
     end
+
     begin
       Order.transaction do
         @carts.each do | cart |
@@ -400,12 +426,16 @@ class CartController < BaseController
           @login_customer.carts.delete_all
           @login_customer.save!
         end
-        @order.save!
+        
+        order_ids = Hash.new
+        @orders.each do |key, order|
+          order.save!
+          order_ids[key] = order.id
+          Notifier::deliver_buying_complete(order)
+        end
         flash[:completed] = true
-        flash[:order_id] = @order.id
-        # メールを送る
-        Notifier::deliver_buying_complete(@order)
-        flash[:googleanalytics_ec] = add_googleanalytics_ec(@order, @order_delivery, @order_details)
+        flash[:order_ids] = order_ids
+        flash[:googleanalytics_ecs] = add_googleanalytics_ecs(@orders, @order_deliveries, @order_details)
         @carts.clear
       end
     rescue => e
@@ -415,7 +445,7 @@ class CartController < BaseController
       redirect_to :action => 'show'
       return
     end
-    redirect_to :action => :finish, :ids => @order_details.map{|o_d| o_d.product_style.product_id}
+    redirect_to :action => :finish, :ids => ids
   end
 
   def finish
@@ -533,6 +563,23 @@ class CartController < BaseController
       end
   end
 
+  def total_points_each_cart(carts)
+    carts.inject(0) do | result, cart |
+      cart.product or next
+      point_rate_product = cart.product.point_granted_rate
+      point_rate_shop = Shop.find(:first).point_granted_rate
+      point_granted_rate = 0
+      unless point_rate_product.blank?
+        point_granted_rate = point_rate_product
+      else 
+        unless point_rate_shop.blank?
+          point_granted_rate = point_rate_shop
+        end
+      end
+      result + cart.price * point_granted_rate / 100 * cart.quantity
+    end
+  end
+
   # 購入時にログイン有無を確認してrenderするフィルタ
   def login_divaricate
     if @login_customer.nil?
@@ -588,6 +635,34 @@ class CartController < BaseController
     end.flatten.uniq.join("\n")
   end
 
+  def init_order_deliveries_for_complete
+    @order_deliveries = Hash.new
+    params[:order_deliveries].each do |key, order_delivery|
+      @order_deliveries[key] = OrderDelivery.new(order_delivery)
+      @order_deliveries[key].set_customer(@login_customer) if @login_customer
+    end
+    @order_details_map = Hash.new
+    @order_deliveries.each do |key, order_delivery|
+      cart = @carts_map[key.to_i]
+      @order_details_map[key] = order_delivery.details_build_from_carts(cart)
+    end
+  end
+  
+  def init_order_deliveries
+    @order_deliveries = Hash.new
+    params[:order_deliveries].each do |key, order_delivery|
+      @order_deliveries[key] = OrderDelivery.new(order_delivery)
+      @order_deliveries[key].set_customer(@login_customer) if @login_customer
+      @order_deliveries[key].payment_id = params[:order_delivery][:payment_id]
+    end
+    @order_details_map = Hash.new
+    @order_deliveries.each do |key, order_delivery|
+      cart = @carts_map[key.to_i]
+      @order_details_map[key] = order_delivery.details_build_from_carts(cart)
+    end
+  end
+
+
   def init_order_delivery
     @order_delivery = OrderDelivery.new(params[:order_delivery])
     @order_delivery.set_customer(@login_customer) if @login_customer
@@ -650,6 +725,16 @@ class CartController < BaseController
     @optional_address.address_detail = order_delivery.deliv_address_detail
   end
 
+  def add_googleanalytics_ecs(orders, deliveries, details_map)
+    googleanalytics_ecs = Array.new
+    orders.each do |key, order|
+      delivery = deliveries[key]
+      details = details_map[key]
+      googleanalytics_ecs << add_googleanalytics_ec(order, delivery, details)
+    end
+    return googleanalytics_ecs
+  end
+
   def add_googleanalytics_ec(order, delivery, details)
     ecommerce = GoogleAnalyticsEcommerce.new
     trans = GoogleAnalyticsTrans.new
@@ -675,7 +760,21 @@ class CartController < BaseController
 
     ecommerce.trans = trans
     
-    flash[:googleanalytics_ec] = ecommerce
+    #flash[:googleanalytics_ec] = ecommerce
+    return ecommerce
+  end
+
+  def select_delivery_trader_with_retailer_id(retailer_id)
+    return DeliveryTrader.find(:all, :conditions => ["retailer_id = ?", retailer_id])
+  end
+
+  def select_delivery_time_with_delivery_trader_id(delivery_trader_id)
+    delivery_times = DeliveryTime.find(:all, :conditions => ["delivery_trader_id = ? and name <> ''", delivery_trader_id], :order => 'position')
+    options = [['指定なし', nil]]
+    options.concat(delivery_times.map do |dt|
+      [dt.name, dt.id]
+    end)
+    return options
   end
 
 end
